package rocketBugsRender;
/*
	DataSet ... the object to hold info read in from file
*/

/*  <RocketBugsRender - for rendering and graphing the output of Zoom (and other programs)>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.Math.*;
import java.awt.*;
import java.io.*;

public class DataSet {
	int [] dims = new int [2];
	DataColumn [] dataTable;
	String fileName;
	String fileInfo;
	String name;
	String [] colNames;
	Pix2DColumn [] pixTable;
	
	// graphical objects
	ButtonList xAxisList, yAxisList, holdYList, colorList, anList;
	
	public DataSet (String fileName) {
		this.fileName = fileName;
		setName();
		loadDataSet(fileName);
		if (!Env.batching) { makeButtonLists();}
	}
	
	public DataSet (int columns, String name) {
		dims[0] = columns;
		colNames = new String[dims[0]];
		dataTable = new DataColumn[dims[0]];
		this.name = name;
		
		if (!Env.batching) { makeButtonLists();}
	}
	
	public DataSet (int columns, String name, String [] nameCols) {
		dims[0] = columns;
		colNames = new String[dims[0]];
		dataTable = new DataColumn[dims[0]];
		this.name = name;
		for (int i=0;i<dims[0];i++) { 
			//talkln (" colNames " + String.valueOf(i) + " is " + String.valueOf(nameCols[i]));
			this.colNames[i] = nameCols[i]; 
		}
		if (!Env.batching) { makeButtonLists();}
	}
	
	public void nullLinks() {
		dims = null;
		for (int i=0;i<dataTable.length;i++) {
			dataTable[i].nullLinks();
			dataTable[i] = null;
		}
		fileName = null;
		fileInfo = null;
		name = null;
		colNames = null;
		pixTable = null;
		xAxisList = null;
		yAxisList = null;
		holdYList = null;
		colorList = null;
		anList = null;
	}
	
	public int getColumnCt() {
		return dims[0];
	}
	
	public String getName() {
		if (name != null) { return name; } else { return ""; }
	}
	
	public void setName () {
		/*int lastDir = fileName.lastIndexOf("/");
		if (lastDir == -1) { 
			name = fileName;
		} else {
			name = fileName.substring(lastDir+1);
		}*/
		File tempFile = new File(fileName);
		name = tempFile.getName();
		// remove "ListeriaInfo" if appended to filename
		int sInd = name.length()-12;
		if (sInd > 0) {
			if (name.substring(sInd).equals("ListeriaInfo")) {
				name = name.substring(0,sInd);
			}
		}
	}
	
	public void setName (String newName) {
		this.name = newName;
	}
	
	public void loadDataSet (String fileName) {
		talk("Loading dataset from " + fileName + " ....");
		dims = FileOps.getDataDims(fileName);
		if (!Env.batching) { Env.allDataColumnsOn(dims[0]); }	// set all columns for reading if not set by batch
		makeDataTable();
		FileOps.getDataNames(fileName, dims[0], dataTable);
		for (int i=0;i<dims[0];i++) { 
			colNames[i] = dataTable[i].name; 
			//talkln (dataTable[i].name + String.valueOf(Env.dataColumnsOn[i]));
		}
		FileOps.getDataVals(fileName, dims, dataTable);
		talkln("Done!");
	}
	
	public void makeButtonLists () {
		makeButtonLists (-1);
	}
	
	public void makeButtonLists (int index) {
		xAxisList = new ButtonList (name, colNames,"JCheckBox",index);
		yAxisList = new ButtonList (name, colNames,"JCheckBox",index);
		holdYList = new ButtonList (name, colNames,"JCheckBox",index);
		colorList = new ButtonList (name, colNames,"JRadioButton",index);
		anList = new ButtonList(name,colNames,"JCheckBox",index);
	}
	
	public void updateButtonLists (String name) {
		xAxisList.addNewButton(name);
		yAxisList.addNewButton(name);
		holdYList.addNewButton(name);
		colorList.addNewButton(name);
		anList.addNewButton(name);
	}
		
	public void makeDataTable () {
		colNames = new String[dims[0]];
		dataTable = new DataColumn[dims[0]];
		for (int i=0;i<dims[0];i++) {
			if (Env.dataColumnsOn[i]) {
				dataTable[i] = new DataColumn(dims[1],this);
			} else {
				dataTable[i] = new DataColumn(1,this);
			}
		}
	}
	
	public void updateFromGUI () {
		for (int i=0;i<dims[0];i++) {
			// x-axis
			dataTable[i].onX = xAxisList.list[i].isSelected(); 
			// y-axis
			dataTable[i].onY = yAxisList.list[i].isSelected();
			// holdY
			dataTable[i].pProps.holdRange = holdYList.list[i].isSelected();
			// colors
			dataTable[i].setColor(colorList.list[i].getForeground());
			// analysis
			dataTable[i].analyze = anList.list[i].isSelected();
		}
	}
	
	public void scaleFields (Dimension lineDim, int[] rowRange) {
		for (int i=0;i<dims[0];i++) {
			dataTable[i].setScale(lineDim, rowRange);
		}
	}
	
	public void makePixTable () {
		// initialize pixTable;
		int numPixCol = 0;
		for (int i=0;i<dims[0];i++) { numPixCol += dataTable[i].getPixColCt(); }
		pixTable = new Pix2DColumn[numPixCol];
		// load pix columns from each DataColumn
		int colCt = 0;
		for (int i=0;i<dims[0];i++) {
			Pix2DColumn [] pixA = dataTable[i].getPixColumns();
			if (pixA == null) { 
				pixTable[colCt] = null;
				colCt++;
			} else {
				for (int j=0;j<pixA.length;j++) {
					pixTable[colCt] = pixA[j];
					colCt++;
				}
			}
		}
	}
	
	public Pix2DColumn[] getAllPix() {
		return pixTable;
	}
	
	public Axis [] getAllYAxes () {
		Axis [] allYAxes = new Axis [dims[0]];
		for (int i=0;i<allYAxes.length;i++) {
			if ((dataTable[i].onY) & (dataTable[i].axis.showAxis)) {
				allYAxes[i] = dataTable[i].axis;
			} else {
				allYAxes[i] = null;
			}
		}
		return allYAxes;
	}
	
	public void reportRates (double deltaT) {
		for (int i=0;i<dims[0];i++) {
			if (dataTable[i].onY) {
				double ave = dataTable[i].range/deltaT;
				talkln (dataTable[i].name + " average = " + String.valueOf(ave));
			}
		}
	}
	
	public void reportAves (double deltaT,DataColumn XC) {
		for (int i=0;i<dims[0];i++) {
			if (dataTable[i].onY) {
				String name = dataTable[i].name;
				if ((name.equals("path.x")) | (name.equals("path.y")) | (name.equals("path.z")) | (name.endsWith("x-y-z D"))) {
					double speed = dataTable[i].rangeSpeed(XC);
					talkln (dataTable[i].name + " range speed = " + String.valueOf(speed));
				} else {
					double ave = dataTable[i].rangeAve();
					talkln (dataTable[i].name + " range average = " + String.valueOf(ave));
				}
			}
		}
		if (dims[0] > 3) {
			if ((dataTable[1].onY) & (dataTable[2].onY) & (dataTable[3].onY)) {
				double xS = dataTable[1].rangeSpeed(XC);
				double yS = dataTable[2].rangeSpeed(XC);
				double zS = dataTable[3].rangeSpeed(XC);
				double xyzS = Math.sqrt(xS*xS + yS*yS + zS*zS);
				talkln (name + ":speed = " + String.valueOf(xyzS));
			}
		}
	}
	
	public void putCol (int index, DataColumn newCol) {
		dataTable[index] = newCol;
		colNames[index] = newCol.getName();
	}
			
	public void addCol (DataColumn newCol) {
		newCol.set = this;
		DataColumn [] tempTable = dataTable;
		String [] tempNames = colNames;
		dataTable = new DataColumn [dims[0]+1];
		colNames = new String [dims[0]+1];
		for (int i=0;i<dims[0];i++) {
			dataTable[i] = tempTable[i];
			colNames[i] = tempNames[i];
		}
		putCol(dims[0],newCol);
		dims[0]++;
		updateButtonLists(newCol.getName());
	}
	
	public void printDataLine (int row) {
		for (int i=0; i<dims[0]; i++) {
			talkln ("Line " + String.valueOf(row) + " , Column " + String.valueOf(i) + " = " + String.valueOf(dataTable[i].data[row]));
		}
	}
	
	private void talk (String info) {
		System.out.print(info);
	}
	
	private void talkln (String info) {
		System.out.println(info);
	}
	
}